/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#define LOG_FRAME_DROPS false

#include "ExpCamera.h"
#include "Transmission/Unreal-Interop/SharedSurfaces/SharedSurfaceProducer.hpp"
#include "Data/SharedTexIds.h"
#include "Data/SdkData.h"
#include "MixCastLog.h"
#include "Utility/MiscUtil.hpp"

#include "CoreMinimal.h"
#include "Engine/World.h"
#include "Kismet/GameplayStatics.h"

#define CHAIN_SIZE 3

class MIXCAST_API FExpFrameSender
{
public:
	FExpFrameSender(AExpCamera* cam)
	{
		this->cam = cam;
		frameDelegate = AExpCamera::OnFrameEnded().AddRaw(this, &FExpFrameSender::HandleFrameUpdated);
	}
	~FExpFrameSender()
	{
		AExpCamera::OnFrameEnded().Remove(frameDelegate);
		if (producer != NULL)
		{
			//producer->PushEndOfFile(1000);
			delete producer;
			producer = NULL;
		}
	}

	void HandleFrameUpdated(AExpCamera* curCam)
	{
		if (curCam != cam)
			return;

		if (producer == NULL) {
			//ugly string conversion!
			std::string s(TCHAR_TO_UTF8(*SharedTexIds::Cameras::ExperienceLayers::Get(cam->Identifier)));
			int len;
			int slength = (int)s.length() + 1;
			len = MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, 0, 0);
			wchar_t* buf = new wchar_t[len];
			MultiByteToWideChar(CP_ACP, 0, s.c_str(), slength, buf, len);
			std::wstring r(buf);
			delete[] buf;

			producer = new SharedSurfaceProducer(r.c_str(), CHAIN_SIZE);
		}
		else {
			producer->EnqueueSurfaceFenceChecks();
		}

		UMixCastFrameCanvas* layersCanvas = cam->GetLayersCanvas();
		if (!layersCanvas->IsValidLowLevel() || !layersCanvas->Resource->TextureRHI)
			return;

		//ugly texture conversion!
		ID3D11Texture2D* srcTex = static_cast<ID3D11Texture2D*>(layersCanvas->Resource->TextureRHI->GetNativeResource());
		if (srcTex == nullptr)
			return;

		FFrameTimer* timer = cam->GetTimer();
		if (!producer->TryPushFrame(srcTex, timer->LastIdealTime(), timer->LastUpdateTime(), timer->LastFrameIndex(), 0))
		{
			if (!consumerMissedLastFrame && firstWriteCompleted && LOG_FRAME_DROPS)
			{
				VirtualCamera* camData = SdkData::GetCameraById(cam->Identifier);
				MIXCAST_WARNING("Compositing process started missing frames at time {%s}",
					*FFrameTimer::GetSMPTE(FFrameTimer::GetMidnightRelativeFrameIndex(timer->LastIdealTime(), camData->renderFramerate), camData->renderFramerate));
			}
			consumerMissedLastFrame = true;
			return;
		}

		if (!firstWriteCompleted && LOG_FRAME_DROPS)
		{
			firstWriteCompleted = true;
			consumerMissedLastFrame = false;

			VirtualCamera* camData = SdkData::GetCameraById(cam->Identifier);
			MIXCAST_LOG("Compositing process started at time {%s}",
				*FFrameTimer::GetSMPTE(FFrameTimer::GetMidnightRelativeFrameIndex(timer->LastIdealTime(), camData->renderFramerate), camData->renderFramerate));
		}

		if (consumerMissedLastFrame && LOG_FRAME_DROPS)
		{
			VirtualCamera* camData = SdkData::GetCameraById(cam->Identifier);
			MIXCAST_LOG("Compositing process back in business at time {%s}",
				*FFrameTimer::GetSMPTE(FFrameTimer::GetMidnightRelativeFrameIndex(timer->LastIdealTime(), camData->renderFramerate), camData->renderFramerate));
		}

		consumerMissedLastFrame = false;
	}

private:
	AExpCamera* cam = NULL;
	FDelegateHandle frameDelegate;

	SharedSurfaceProducer* producer = NULL;

	bool firstWriteCompleted = false;
	bool consumerMissedLastFrame = false;
};
