/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"

#include "MixCastLog.h"
#include "Utility/MathUtil.hpp"
#include "Transmission/Generated/Math_types.h"
#include "Thrift/TOutput.h"

#include <string>

using namespace apache::thrift;

namespace mixcast { namespace thrift {

class MIXCAST_API ThriftUtil
{
public:
	// Creates the Windows pipe name with server prefix.
	// https://docs.microsoft.com/en-us/windows/desktop/ipc/pipe-names
	static std::string GetFullPipeName(const std::string& PipeName)
	{
		return R"(\\.\pipe\)" + PipeName;
	}

	static void SetUpLogCallback()
	{
		GlobalOutput.setOutputFunction([](const char* Message)
		{
			const FString String = FString(UTF8_TO_TCHAR(Message));
			MIXCAST_LOG("[Thrift] %s", *String);
		});
	}

	static FVector ThriftPositionToUnreal(const Vector3 &UnityPosition)
	{
		return MathUtil::UnityPositionToUnreal({ (float)UnityPosition.x, (float)UnityPosition.y, (float)UnityPosition.z });
	}
	static FVector ThriftScaleToUnreal(const Vector3 &UnityScale)
	{
		return MathUtil::UnityVectorToUnreal({ (float)UnityScale.x, (float)UnityScale.y, (float)UnityScale.z });
	}
	static FQuat ThriftQuatToUnreal(const Quaternion &UnityRotation)
	{
		return MathUtil::UnityRotationToUnreal({ (float)UnityRotation.x, (float)UnityRotation.y, (float)UnityRotation.z, (float)UnityRotation.w });
	}
	static Vector3 UnrealVectorToThrift(const FVector &UnrealPosition)
	{
		FVector unity = MathUtil::UnrealPositionToUnity(UnrealPosition);
		Vector3 thrift;
		thrift.x = unity.X;
		thrift.y = unity.Y;
		thrift.z = unity.Z;
		return thrift;
	}
	static Quaternion UnrealQuatToThrift(const FQuat &UnrealRotation)
	{
		FQuat unity = MathUtil::UnrealRotationToUnity(UnrealRotation);
		Quaternion thrift;
		thrift.x = unity.X;
		thrift.y = unity.Y;
		thrift.z = unity.Z;
		thrift.w = unity.W;
		return thrift;
	}
};

} }
